package java2dexperimente;

// Demo fr Java2D
// 2001-10-16
// Jrn Loviscach

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;

public class Experiment1 {
    public static void main( String[] args ) {
        new DemoWindow();
    }
}

class DemoWindow {
  private JFrame myFrame = new JFrame("Experiment1"); //das Fenster
  private Container myPane = myFrame.getContentPane(); //seine Inhaltsflche
  private JSlider mySlider = new JSlider(JSlider.VERTICAL, 0, 255, 128); //ein Schieberegler
  private DoodlePanel myDoodlePanel = new DoodlePanel(); //eine Zeichenflche
  private JPanel myOptions = new JPanel(); //Flche fr die Wahlknpfe
  private ButtonGroup myButtonGroup = new ButtonGroup(); //lst die Wahlknpfe gegenseitig aus

  DemoWindow() {
    myFrame.addWindowListener (
      new WindowAdapter() {
        public void windowClosing(WindowEvent e)
        {
          System.exit(0);
        }
      }
    );

    //Das Folgende wird jedem Knopf zugewiesen, damit etwas passiert,
    //wenn man einen Knopf drckt.
    ActionListener myButtonListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) { //Andere Option wurde gewhlt.
        //entsprechende Variable ndern:
        myDoodlePanel.setOption(myButtonGroup.getSelection().getActionCommand());

        myDoodlePanel.repaint();
      }
    };

    JRadioButton radioButton; // Hilfsvariable
    radioButton = new JRadioButton("Fllung/Strichstrke");
    //zu logischer Gruppe hinzufgen (gegenseitiges Auslsen):
    myButtonGroup.add(radioButton);
    //zu grafischer Gruppe hinzufgen:
    myOptions.add(radioButton);
    //Kennung, mit der wir den Knopf als Ereignis herausfischen:
    radioButton.setActionCommand("Fllung");
    //Dieser Knopf hier ist zu Beginn gedrckt:
    radioButton.setSelected(true);
    //Ereignisse per myButtonListener melden:
    radioButton.addActionListener(myButtonListener);

    //und dasselbe Spiel (RadioButton) noch zwei Mal:

    radioButton = new JRadioButton("Transformation/Compositing");
    myButtonGroup.add(radioButton);
    myOptions.add(radioButton);
    radioButton.setActionCommand("Transformation");
    radioButton.addActionListener(myButtonListener);

    radioButton = new JRadioButton("Verlauf/Clipping");
    myButtonGroup.add(radioButton);
    myOptions.add(radioButton);
    radioButton.setActionCommand("Verlauf");
    radioButton.addActionListener(myButtonListener);

    //Anfangsstellung der Wahlknpfe in entsprechende Variable bertragen
    myDoodlePanel.setOption(myButtonGroup.getSelection().getActionCommand());

    myFrame.setSize(800, 400);
    myFrame.setLocation(50, 50);

    //automatische Anordnung auf dem Bildschirm steuern
    myPane.setLayout(new GridBagLayout());

    //Objekt, in dem man die gewnschte Anordnung der Elemente ablegt.
    //Wird hier mehrfach benutzt, mit jeweils verschiedenen Wnschen.
    GridBagConstraints myConstraints = new GridBagConstraints();

    //Positionierungswnsche fr Zeichenflche
    myConstraints.fill = GridBagConstraints.BOTH;
    //Zeichenflche soll sich im Fenster ausdehnen
    myConstraints.weightx = 1.0;
    myConstraints.weighty = 1.0;
    //Verhltnis, in dem sich die Zeichenflche zu anderen Elementen ausdehnt
    myConstraints.gridx = 0;
    myConstraints.gridy = 0;
    //Zeichenflche sitzt links in der Zeile null.
    myPane.add(myDoodlePanel, myConstraints);

    //Positionierungswnsche fr Wahlknpfe
    myConstraints.weightx = 0.0;
    myConstraints.weighty = 0.0;
    //Panel mit Wahlknpfen nur so breit, wie es sein muss
    myConstraints.gridx = 0;
    myConstraints.gridy = 1;
    myConstraints.gridwidth = 2;
    //Wahlknpfe sitzen links in der Zeile eins und gehen ber beide Spalten.
    myPane.add(myOptions, myConstraints);

    //Positionierungswnsche fr Schieberegler
    myConstraints.weightx = 0.0;
    myConstraints.weighty = 0.0;
    //Schieberegler nur so breit, wie er sein muss
    myConstraints.gridx = 1;
    myConstraints.gridy = 0;
    myConstraints.gridwidth = 1; //zurckstellen, weil oben verstellt
    //Schieberegler sitzt rechts in der Zeile null.
    myPane.add(mySlider, myConstraints);

    //Vier lange Teilstriche am Schieberegler (256/4 = 64).
    //Ganz oben beim Wert 256 se der fnfte lange Strich,
    //aber der Schieberegler geht nur bis 255 statt bis 256.
    mySlider.setMajorTickSpacing(64);

    //Auf jeweils halber Distanz einen kurzen Teilstrich:
    mySlider.setMinorTickSpacing(32);

    //Und sehen wollen wir die Striche auch:
    mySlider.setPaintTicks(true);

    //Ergebnisse per anonymen ChangeListener melden:
    mySlider.addChangeListener(
      new ChangeListener() {
        public void stateChanged(ChangeEvent e) { //Schieberegler wurde verstellt.
          //entsprechende Variable ndern:
          myDoodlePanel.setParameter(mySlider.getValue());

          myDoodlePanel.repaint();
        }
      }
    );

    //Anfangsposition des Reglers in entsprechende Variable bertragen
    myDoodlePanel.setParameter(mySlider.getValue());

    //Fenster endlich *schwitz* anzeigen
    myFrame.setVisible(true);
  }
}

class DoodlePanel extends JPanel {
  private int parameter; //beeinflusst Formen und/oder Farben
  private String option; //Auswahl der Demo-Routine

  void setParameter(int p) {
    parameter = p;
  }

  void setOption(String o) {
    option = o;
  }

  public void paint(Graphics g) {
    super.paint(g);
    Graphics2D g2 = (Graphics2D) g;

    //Kantenglttung an
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    if (option.equals("Fllung")) { //Wahlknopf "Fllung" ist gedrckt
      //Strichstrke von Schieberegler abhngig machen:
      g2.setStroke(new BasicStroke(1+parameter/32));

      g2.setPaint(Color.white); //vordefinierte Farbe
      g2.fillOval(20, 30, 200, 300); //Ellipsenflche ausmalen
      g2.setPaint(Color.red); //vordefinierte Farbe
      g2.drawOval(20, 30, 200, 300); //Randkurve der Ellipse zeichnen
      //Erst fill, dann draw, sonst wird der innere Rand des Umrisses bermalt!

      //Eine Ellipse erzeugen und als Vektorgrafikobjekt zwischenspeichern:
      Shape myCurve = new Ellipse2D.Double(50, 50, 400, 200);

      //Farbe auf reglergesteuerten Phantasiewert stellen:
      g2.setPaint(new Color(parameter, 128, parameter/2));

      //ausmalen
      g2.fill(myCurve);

      g2.setPaint(Color.black);

      //Umriss zeichnen
      g2.draw(myCurve);

    } else if (option.equals("Transformation")) { //Wahlknopf "Transformation" ist gedrckt
      //Eine Ellipse erzeugen und als Vektorgrafikobjekt zwischenspeichern.
      Shape myCurve = new Ellipse2D.Double(200, 50, 400, 200);

      for(int i=0; i<5; i++) { //Ellipse gleich fnfmal ausgeben.
        //in jedem Durchlauf der Schleife eine andere Farbe
        g2.setPaint(new Color(30*i, 128, 255-40*i));

        //Alle (!) kommenden Zeichenbefehle werden gedreht ausgefhrt,
        //z.B. bei rotate(Math.PI/2) um 90 im Uhrzeigersinn gedreht.
        g2.rotate(Math.PI/40*i);

        //Alle kommenden Zeichenbefehle werden verschoben ausgefhrt,
        //bei translate(x,y) um x nach rechts und y nach unten.
        g2.translate(parameter*i/5.0, -parameter*i/5.0);

        //Alles Kommende um den angegebenen Alphawert 0...1 halbtransparent zeichnen;
        //hngt hier vom Schleifenindex ab.
        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float) ((5-i)/5.0)));

        //Flche der zwischengespeicherten Ellipse ausmalen.
        g2.fill(myCurve);
      }

    } else if (option.equals("Verlauf")) { //Wahlknopf "Verlauf" ist gedrckt
      for( int x=100; x<700; x+=50) {
        for( int y=50; y<350; y+=30) {
          //Zeichen "ABC" an Stelle x, y zeichnen
          g2.drawString("ABC", x, y);
        }
      }

      //Zeichnung ab hier auf Ellipse beschrnken (Clipping Path).
      //Die wird per Schieberegler bewegt.
      g2.clip(new Ellipse2D.Double(150+1.5*parameter, 20, 200, 250));

      //Fllung mit Farbverlauf: Am Punkt (0,10) liegt Rot, bei (500,400) Blau.
      g2.setPaint(new GradientPaint(0, 10, Color.red, 500, 400, Color.blue));

      //Ellipsenflche ausmalen.
      g2.fillOval(200, 50, 400, 200);
    }
  }
}